"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.locateFiles = locateFiles;
exports.parseColor = parseColor;
exports.loadSVG = void 0;

var _path = _interopRequireDefault(require("path"));

var _cheerio = _interopRequireDefault(require("cheerio"));

var _debug = _interopRequireDefault(require("debug"));

var _expandTilde = _interopRequireDefault(require("expand-tilde"));

var _fastGlob = _interopRequireDefault(require("fast-glob"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const debug = (0, _debug.default)('sqip');

const loadSVG = svg => {
  return _cheerio.default.load(svg, {
    normalizeWhitespace: true,
    xmlMode: true
  });
};

exports.loadSVG = loadSVG;

async function locateFiles(input) {
  const enhancedInput = (0, _expandTilde.default)(input);
  let globPattern = enhancedInput;

  try {
    const stat = await _fsExtra.default.lstat(enhancedInput);

    if (stat.isFile()) {
      debug(`input ${input} is a file. Skip file search.`);
      return [enhancedInput];
    }

    if (stat.isDirectory()) {
      debug(`input ${input} is a directory. Enhancing with * to match all files.`);
      globPattern = `${_path.default.resolve(enhancedInput)}/*`;
    }
  } catch (err) {
    if (err.code !== 'ENOENT') {
      throw err;
    }
  } // Find all files matching the enhanced glob


  const files = await (0, _fastGlob.default)(globPattern, {
    onlyFiles: true,
    extglob: true,
    absolute: true
  }); // Test if files are found

  if (!files.length) {
    throw new Error(`Unable to find any files via ${globPattern}. Make sure the file exists.

If you are using globbing patterns, the following features are supported:

https://github.com/micromatch/micromatch#matching-features`);
  }

  return files;
}

function parseColor({
  palette,
  color
}) {
  return color in palette ? palette[color].getHex() : color;
}