"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
var _exportNames = {
  resolvePlugins: true,
  SqipPlugin: true
};
exports.resolvePlugins = resolvePlugins;
exports.default = sqip;
exports.SqipPlugin = void 0;

var _path = _interopRequireDefault(require("path"));

var _debug = _interopRequireDefault(require("debug"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _probeImageSize = _interopRequireDefault(require("probe-image-size"));

var _nodeVibrant = _interopRequireDefault(require("node-vibrant"));

var _sharp = _interopRequireDefault(require("sharp"));

var _termImg = _interopRequireDefault(require("term-img"));

var _cliTable = _interopRequireDefault(require("cli-table3"));

var _chalk = _interopRequireDefault(require("chalk"));

var _helpers = require("./helpers");

Object.keys(_helpers).forEach(function (key) {
  if (key === "default" || key === "__esModule") return;
  if (Object.prototype.hasOwnProperty.call(_exportNames, key)) return;
  if (key in exports && exports[key] === _helpers[key]) return;
  Object.defineProperty(exports, key, {
    enumerable: true,
    get: function () {
      return _helpers[key];
    }
  });
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const debug = (0, _debug.default)('sqip');
const mainKeys = ['originalWidth', 'originalHeight', 'width', 'height', 'type'];
const paletteKeys = ['Vibrant', 'DarkVibrant', 'LightVibrant', 'Muted', 'DarkMuted', 'LightMuted']; // Resolves plugins based on a given config
// Array of plugin names or config objects, even mixed.

async function resolvePlugins(plugins) {
  return Promise.all(plugins.map(async plugin => {
    if (typeof plugin === 'string') {
      plugin = {
        name: plugin
      };
    }

    const {
      name
    } = plugin;

    if (!name) {
      throw new Error(`Unable to read plugin name from:\n${JSON.stringify(plugin, null, 2)}`);
    }

    const moduleName = name.indexOf('sqip-plugin-') !== -1 ? name : `sqip-plugin-${name}`;

    try {
      debug(`Loading ${moduleName}`);
      const Plugin = await Promise.resolve(`${moduleName}`).then(s => _interopRequireWildcard(require(s)));
      return _objectSpread(_objectSpread({}, plugin), {}, {
        Plugin: Plugin.default
      });
    } catch (err) {
      if (err.code === 'MODULE_NOT_FOUND') {
        throw new Error(`Unable to load plugin "${moduleName}". Try installing it via:\n\n npm install ${moduleName}`);
      }

      throw err;
    }
  }));
}

async function processFile({
  buffer,
  outputFileName,
  config
}) {
  const {
    output,
    silent,
    parseableOutput
  } = config;
  const result = await processImage({
    buffer,
    config
  });
  const {
    content,
    metadata
  } = result;
  let outputPath;
  debug(`Processed ${outputFileName}`); // Write result svg if desired

  if (output) {
    try {
      // Test if output path already exists
      const stats = await _fsExtra.default.stat(output); // Throw if it is a file and already exists

      if (!stats.isDirectory()) {
        throw new Error(`File ${output} already exists. Overwriting is not yet supported.`);
      }

      outputPath = _path.default.resolve(output, `${outputFileName}.svg`);
    } catch (err) {
      // Output directory or file does not exist. We will create it later on.
      outputPath = output;
    }

    debug(`Writing ${outputPath}`);
    await _fsExtra.default.writeFile(outputPath, content);
  } // Gather CLI output information


  if (!silent) {
    if (outputPath) {
      console.log(`Stored at: ${outputPath}`);
    } // Generate preview


    if (!parseableOutput) {
      // Convert to png for image preview
      const preview = await (0, _sharp.default)(Buffer.from(content)).png().toBuffer();

      try {
        (0, _termImg.default)(preview, {
          fallback: () => {
            // SVG results can still be outputted as string
            if (metadata.type === 'svg') {
              console.log(content.toString());
              return;
            } // No fallback preview solution yet for non-svg files.


            console.log(`Unable to render a preview for ${metadata.type} files on this machine. Try using https://iterm2.com/`);
          }
        });
      } catch (err) {
        if (err.name !== 'UnsupportedTerminalError') {
          throw err;
        }
      }
    } // Metadata


    const tableConfig = parseableOutput && {
      chars: {
        top: '',
        'top-mid': '',
        'top-left': '',
        'top-right': '',
        bottom: '',
        'bottom-mid': '',
        'bottom-left': '',
        'bottom-right': '',
        left: '',
        'left-mid': '',
        mid: '',
        'mid-mid': '',
        right: '',
        'right-mid': '',
        middle: ' '
      },
      style: {
        'padding-left': 0,
        'padding-right': 0
      }
    }; // Figure out which metadata keys to show

    const allKeys = [...mainKeys, 'palette'];

    const restMetadata = _objectSpread({}, metadata);

    allKeys.forEach(k => delete restMetadata[k]);
    const mainTable = new _cliTable.default(tableConfig);
    mainTable.push(mainKeys);
    mainTable.push(mainKeys.map(key => metadata[key]));
    console.log(mainTable.toString()); // Show color palette

    const paletteTable = new _cliTable.default(tableConfig);
    paletteTable.push(paletteKeys);
    paletteTable.push(paletteKeys.map(key => metadata.palette[key].getHex()).map(hex => _chalk.default.hex(hex)(hex)));
    console.log(paletteTable.toString());
    Object.keys(restMetadata).forEach(key => {
      console.log(_chalk.default.bold(`${key}:`));
      console.log(restMetadata[key]);
    });
  }

  return result;
}

async function processImage({
  buffer,
  config
}) {
  const originalSizes = _probeImageSize.default.sync(buffer);

  const vibrant = _nodeVibrant.default.from(buffer);

  const palette = await vibrant.quality(0).getPalette();
  let metadata = {
    originalWidth: originalSizes.width,
    originalHeight: originalSizes.height,
    palette
  }; // Load plugins

  const plugins = await resolvePlugins(config.plugins); // Interate through plugins and apply them to last returned image

  if (config.width > 0) {
    // Resize to desired output width
    buffer = await (0, _sharp.default)(buffer).resize(config.width).toBuffer();
    const resizedMetadata = await (0, _sharp.default)(buffer).metadata();
    metadata.width = resizedMetadata.width;
    metadata.height = resizedMetadata.height;
  } else {
    // Fall back to original size, keep image as is
    metadata.width = originalSizes.width;
    metadata.height = originalSizes.height;
  }

  for (const {
    name,
    options: pluginOptions,
    Plugin
  } of plugins) {
    debug(`Construct ${name}`);
    const plugin = new Plugin({
      sqipConfig: config,
      pluginOptions,
      metadata
    });
    debug(`Apply ${name}`);
    buffer = await plugin.apply(buffer);
  }

  return {
    content: buffer,
    metadata
  };
}

async function sqip(options) {
  // Build configuration based on passed options and default options
  const defaultOptions = {
    plugins: [{
      name: 'primitive',
      options: {
        numberOfPrimitives: 8,
        mode: 0
      }
    }, 'blur', 'svgo', 'data-uri'],
    width: 300,
    parseableOutput: false,
    silent: true
  };
  const config = Object.assign({}, defaultOptions, options);
  const {
    input,
    outputFileName,
    parseableOutput,
    silent
  } = config;

  if (parseableOutput) {
    _chalk.default.enabled = false;
  } // Validate configuration


  if (!input) {
    throw new Error('Please provide an input image, e.g. sqip({ input: "input.jpg" })');
  } // If input is a Buffer


  if (Buffer.isBuffer(input)) {
    return processFile({
      buffer: input,
      outputFileName,
      config
    });
  }

  const files = await (0, _helpers.locateFiles)(input);
  debug('Found files:');
  debug(files); // Test if all files are accessable

  for (const file of files) {
    try {
      debug('check file ' + file);
      await _fsExtra.default.access(file, _fsExtra.default.constants.R_OK);
    } catch (err) {
      throw new Error(`Unable to read file ${file}`);
    }
  } // Iterate over all files


  const results = [];

  for (const filePath of files) {
    // Apply plugins to files
    if (!silent) {
      console.log(`Processing: ${filePath}`);
    } else {
      debug(`Processing ${filePath}`);
    }

    const buffer = await _fsExtra.default.readFile(filePath);
    const result = await processFile({
      buffer,
      outputFileName: outputFileName || _path.default.parse(filePath).name,
      config
    });
    results.push(result);
  }

  debug(`Finished`); // Return as array when input was array or results is only one file

  if (Array.isArray(input) || results.length === 0) {
    return results;
  }

  return results[0];
}

class SqipPlugin {
  constructor({
    sqipConfig,
    metadata
  }) {
    this.sqipConfig = sqipConfig || {};
    this.metadata = metadata || {};
  }

}

exports.SqipPlugin = SqipPlugin;